@@ -42,9 +42,9 @@
 #define CRYPTOKI_EXPORTS
 
 #include "attrs.h"
-#include "debug.h"
 #include "hash.h"
 #include "library.h"
+#include "path.h"
 #include "pkcs11x.h"
 #include "test-data.h"
 #include "token.h"
@@ -82,7 +82,9 @@ setup (CuTest *cu)
 	CuAssertTrue (cu, rv == CKR_OK);
 
 	memset (&args, 0, sizeof (args));
-	paths = SRCDIR "/input:" SRCDIR "/files/self-signed-with-ku.der:" SRCDIR "/files/thawte.pem";
+	paths = SRCDIR "/input" P11_PATH_SEP \
+		SRCDIR "/files/self-signed-with-ku.der" P11_PATH_SEP \
+		SRCDIR "/files/thawte.pem";
 	if (asprintf (&arguments, "paths='%s'", paths) < 0)
 		CuAssertTrue (cu, false && "not reached");
 	args.pReserved = arguments;
@@ -205,7 +207,10 @@ test_get_token_info (CuTest *cu)
 	CuAssertTrue (cu, rv == CKR_OK);
 
 	memset (&args, 0, sizeof (args));
-	args.pReserved = "paths='" SYSCONFDIR "/input:" DATADIR "/files/blah:" "/some/other/path/the-basename'";
+	args.pReserved = "paths='" \
+		SYSCONFDIR "/input" P11_PATH_SEP \
+		DATADIR "/files/blah" P11_PATH_SEP \
+		"/some/other/path/the-basename'";
 	args.flags = CKF_OS_LOCKING_OK;
 
 	rv = module->C_Initialize (&args);
@@ -633,6 +638,46 @@ test_session_find (CuTest *cu)
 	teardown (cu);
 }
 
+static void
+test_session_find_no_attr (CuTest *cu)
+{
+	CK_ATTRIBUTE original[] = {
+		{ CKA_CLASS, &data, sizeof (data) },
+		{ CKA_LABEL, "yay", 3 },
+		{ CKA_VALUE, "eight", 5 },
+		{ CKA_INVALID }
+	};
+
+	CK_ATTRIBUTE match[] = {
+		{ CKA_COLOR, "blah", 4 },
+		{ CKA_INVALID }
+	};
+
+	CK_SESSION_HANDLE session;
+	CK_OBJECT_HANDLE handle;
+	CK_OBJECT_HANDLE check;
+	CK_ULONG count;
+	CK_RV rv;
+
+	setup (cu);
+
+	rv = test.module->C_OpenSession (test.slots[0], CKF_SERIAL_SESSION, NULL, NULL, &session);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+
+	rv = test.module->C_CreateObject (session, original, 3, &handle);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+
+	rv = test.module->C_FindObjectsInit (session, match, 1);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+	rv = test.module->C_FindObjects (session, &check, 1, &count);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+	CuAssertIntEquals (cu, 0, count);
+	rv = test.module->C_FindObjectsFinal (session);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+
+	teardown (cu);
+}
+
 static void
 test_lookup_invalid (CuTest *cu)
 {
@@ -867,6 +912,93 @@ test_find_serial_der_decoded (CuTest *cu)
 	teardown (cu);
 }
 
+static void
+test_find_serial_der_mismatch (CuTest *cu)
+{
+	CK_OBJECT_CLASS nss_trust = CKO_NSS_TRUST;
+
+	CK_ATTRIBUTE object[] = {
+		{ CKA_CLASS, &nss_trust, sizeof (nss_trust) },
+		{ CKA_SERIAL_NUMBER, "\x02\x03\x01\x02\x03", 5 },
+		{ CKA_INVALID }
+	};
+
+	CK_ATTRIBUTE match[] = {
+		{ CKA_SERIAL_NUMBER, NULL, 0 },
+		{ CKA_CLASS, &nss_trust, sizeof (nss_trust) },
+		{ CKA_INVALID }
+	};
+
+	CK_SESSION_HANDLE session;
+	CK_OBJECT_HANDLE handle;
+	CK_OBJECT_HANDLE check;
+	CK_ULONG count;
+	CK_RV rv;
+
+	setup (cu);
+
+	rv = test.module->C_OpenSession (test.slots[0], CKF_SERIAL_SESSION, NULL, NULL, &session);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+
+	rv = test.module->C_CreateObject (session, object, 2, &handle);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+
+	/* Do a find with a null serial number, no match */
+	rv = test.module->C_FindObjectsInit (session, match, 2);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+	rv = test.module->C_FindObjects (session, &check, 1, &count);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+	CuAssertIntEquals (cu, 0, count);
+	rv = test.module->C_FindObjectsFinal (session);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+
+	/* Do a find with a wrong length, no match */
+	match[0].pValue = "at";
+	match[0].ulValueLen = 2;
+	rv = test.module->C_FindObjectsInit (session, match, 2);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+	rv = test.module->C_FindObjects (session, &check, 1, &count);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+	CuAssertIntEquals (cu, 0, count);
+	rv = test.module->C_FindObjectsFinal (session);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+
+	/* Do a find with a right length, wrong value, no match */
+	match[0].pValue = "one";
+	match[0].ulValueLen = 3;
+	rv = test.module->C_FindObjectsInit (session, match, 2);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+	rv = test.module->C_FindObjects (session, &check, 1, &count);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+	CuAssertIntEquals (cu, 0, count);
+	rv = test.module->C_FindObjectsFinal (session);
+	CuAssertIntEquals (cu, CKR_OK, rv);
+
+	teardown (cu);
+}
+
+static void
+test_login_logout (CuTest *cu)
+{
+	CK_SESSION_HANDLE session;
+	CK_RV rv;
+
+	setup (cu);
+
+	rv = test.module->C_OpenSession (test.slots[0], CKF_SERIAL_SESSION, NULL, NULL, &session);
+	CuAssertTrue (cu, rv == CKR_OK);
+
+	/* Just testing our stubs for now */
+
+	rv = test.module->C_Login (session, CKU_USER, NULL, 0);
+	CuAssertTrue (cu, rv == CKR_USER_TYPE_INVALID);
+
+	rv = test.module->C_Logout (session);
+	CuAssertTrue (cu, rv == CKR_USER_NOT_LOGGED_IN);
+
+	teardown (cu);
+}
+
 int
 main (void)
 {
@@ -876,7 +1008,6 @@ main (void)
 
 	putenv ("P11_KIT_STRICT=1");
 	p11_library_init ();
-	p11_debug_init ();
 
 	SUITE_ADD_TEST (suite, test_get_slot_list);
 	SUITE_ADD_TEST (suite, test_get_slot_info);
@@ -890,10 +1021,13 @@ main (void)
 	SUITE_ADD_TEST (suite, test_setattr_token);
 	SUITE_ADD_TEST (suite, test_session_object);
 	SUITE_ADD_TEST (suite, test_session_find);
+	SUITE_ADD_TEST (suite, test_session_find_no_attr);
 	SUITE_ADD_TEST (suite, test_session_copy);
 	SUITE_ADD_TEST (suite, test_session_remove);
 	SUITE_ADD_TEST (suite, test_session_setattr);
 	SUITE_ADD_TEST (suite, test_find_serial_der_decoded);
+	SUITE_ADD_TEST (suite, test_find_serial_der_mismatch);
+	SUITE_ADD_TEST (suite, test_login_logout);
 
 	CuSuiteRun (suite);
 	CuSuiteSummary (suite, output);
